# -*- coding: utf-8 -*-
## Used to install ExternalSiteCatalog content
## Copyright (C)2005 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
"""
This module is used by portal_quickinstaller to install product
"""

__docformat__ = 'restructuredtext'

# Python imports
from StringIO import StringIO

# CMF
from Products.CMFCore.utils import getToolByName
from Products.CMFCore import permissions as CCP

# Archetypes
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.Archetypes.public import listTypes

# Products import
from Products.ExternalSiteCatalog.config import PROJECTNAME, GLOBALS, TOOLNAME
from Products.ExternalMethod.ExternalMethod import manage_addExternalMethod

# Configlets
externalsitecatalog_management_configlet = {
    'id': 'externalsitecatalog_management',
    'appId': PROJECTNAME,
    'name': 'External Site Catalog',
    'action': 'string:$portal_url/prefs_externalsitecatalog',
    'category': 'Products',
    'permission': (CCP.ManagePortal,),
    'imageUrl': 'externalsitecatalog_icon.gif',
    }
    
types_to_hide = (TOOLNAME,)

externalsitecatalog_portlets = ('here/external_catalog_search_form_portlet/macros/portlet',)

def install_portlet(self, out):
    """Install portlet in right slots"""
    
    utool = getToolByName(self, 'portal_url')
    portal = utool.getPortalObject()
    portlets = list(portal.getProperty('right_slots'))
    
    for portlet in externalsitecatalog_portlets:
        if portlet not in portlets:
            portlets.append(portlet)
    
    portal._updateProperty('right_slots', portlets)
 
    print >>out, 'ExternalSiteCatalog portlet installed.\n'

def uninstall_portlet(self, out):
    """Install portlet in right slots"""
    
    utool = getToolByName(self, 'portal_url')
    portal = utool.getPortalObject()
    portlets = list(portal.getProperty('right_slots'))
    
    for portlet in externalsitecatalog_portlets:
        if portlet in portlets:
            portlets.remove(portlet)
    
    portal._updateProperty('right_slots', portlets)
    
    print >>out, 'ExternalSiteCatalog portlet uninstalled.\n'

def install(self, reinstall=0):
    """Install ExternalSiteCatalog product"""
    
    out = StringIO()

    # Install portlet
    install_portlet(self, out)
    
    # Install types
    typeInfo = listTypes(PROJECTNAME)
    installTypes(self, out,
                 typeInfo,
                 PROJECTNAME)

    # Install skin
    install_subskin(self, out, GLOBALS)

    # inner content setup

    # no wf chain
    pw = getToolByName(self, 'portal_workflow')
    
    # Update portal properties
    ptool = getToolByName(self, 'portal_properties')
    
    # Update site_properties.types_not_searched property
    sprops = ptool.site_properties
    new_types = list(sprops.types_not_searched)
    for type_id in types_to_hide:
        if type_id in new_types:
            continue
        new_types.append(type_id)
    sprops.types_not_searched = new_types
    
    # Update site_properties.metaTypesNotToList property
    nprops = ptool.navtree_properties
    new_types = list(nprops.metaTypesNotToList)
    for type_id in types_to_hide:
        if type_id in new_types:
            continue
        new_types.append(type_id)
    nprops.metaTypesNotToList = new_types
    
    # Install tool
    atool = getattr(self, TOOLNAME, None)
    if atool is None:
        self.manage_addProduct[PROJECTNAME].manage_addTool('ExternalSiteCatalog')
    
    # Install configlet
    cptool = getToolByName(self, 'portal_controlpanel')
    try:
        cptool.registerConfiglet(**externalsitecatalog_management_configlet)
    except:
        pass

    mtool = getToolByName(self, 'portal_maintenance')
    scriptsholder = mtool.scripts

    if not 'crawlExternalSite' in scriptsholder.objectIds():
        # Plone Catalog update script
        manage_addExternalMethod(scriptsholder,'crawlExternalSite',
            'Crawl external sites',
            PROJECTNAME+'.MaintenanceToolbox',
            'crawlExternalSite')

    out.write('Installation completed.\n')
    return out.getvalue()

def uninstall(self):
    """Uninstall ExternalSiteCatalog product"""
    
    out = StringIO()

    # Uninstall portlet
    uninstall_portlet(self, out)
    
    # Update portal properties
    ptool = getToolByName(self, 'portal_properties')
    
    # Update site_properties.types_not_searched property
    sprops = ptool.site_properties
    new_types = [x for x in sprops.types_not_searched if x not in types_to_hide]
    sprops.types_not_searched = new_types
    
    # Update site_properties.metaTypesNotToList property
    nprops = ptool.navtree_properties
    new_types = [x for x in nprops.metaTypesNotToList if x not in types_to_hide]
    nprops.metaTypesNotToList = new_types

    # Uninstall configlets
    try:
        cptool = getToolByName(self, 'portal_controlpanel')
        cptool.unregisterApplication(PROJECTNAME)
    except:
        pass
    
    out.write('Uninstallation completed.\n')
    return out.getvalue()
    
